package com.starcharge.wios.controller;

import com.alibaba.fastjson.JSONObject;
import com.ihidea.component.api.v2.BaseResponse;
import com.ihidea.core.support.exception.ServiceException;
import com.ihidea.core.support.session.SessionInfo;
import com.ihidea.core.util.HttpClientUtils;
import com.ihidea.core.util.XMLUtilsEx;
import com.starcharge.warehouse.query.MaterialsQuery;
import com.starcharge.wios.auth.filter.LoginFilter;
import com.starcharge.wios.auth.service.TokenService;
import com.starcharge.wios.convert.RejectConvert;
import com.starcharge.wios.dao.entity.Delivery;
import com.starcharge.wios.dao.entity.Reject;
import com.starcharge.wios.dao.mappers.DeliveryMapper;
import com.starcharge.wios.dao.mappers.RejectMapper;
import com.starcharge.wios.dto.DeliveryUpdateDTO;
import com.starcharge.wios.enums2.LogisticStatus;
import com.starcharge.wios.enums2.TauxiliarymaterialStatusEnum;
import com.starcharge.wios.service.CommonUpdateService;
import com.starcharge.wios.service.SnowFlakeService;
import com.starcharge.wios.utils.ExcelUtils;
import com.starcharge.wios.utils.SpringContextUtil;
import com.starcharge.wios.validation.CreateEntityGroup;
import com.starcharge.wios.validation.ParamsValidate;
import com.starcharge.wios.validation.UpdateEntityGroup;
import com.starcharge.wios.vo.RejectAddVo;
import com.starcharge.wios.vo.RejectCheckVo;
import com.starcharge.wios.vo.RejectExcelVo;
import io.netty.util.internal.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import lombok.var;
import org.apache.http.Header;
import org.apache.http.HeaderElement;
import org.apache.http.ParseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpHeaders;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @projectName:xr-wios
 * @packageName:com.starcharge.wios.controller
 * @Description:TODO
 * @Author:luyincheng
 * @CreateDate: 2023/10/819:15
 */
@RestController
@RequestMapping("api/reject")
@Api(tags = "退货管理接口")
@Slf4j
public class RejectController {
    @Autowired
    private TokenService tokenService;
    @Autowired
    private RejectMapper rejectMapper;
    @Autowired
    private CommonUpdateService commonUpdateService;
    @Autowired
    private SnowFlakeService snowFlakeService;
    @Autowired
    private DeliveryMapper deliveryMapper;
    @Value("${warehouseUrl.increaseUrl}")
    private String increaseUrl;
    @Value("${warehouseUrl.reduceUrl}")
    private String reduceUrl;


   /* @PostMapping("/add")
    @ApiOperation(value = "申请退货")
    @ParamsValidate
    public BaseResponse<Integer> AddReject(@RequestBody @Validated(CreateEntityGroup.class) RejectAddVo rejectAddVo, BindingResult bindingResult){
        if (rejectAddVo.getInstallOrder()==null){
            throw new RuntimeException("安装单号不能为空");
        }
        //已经申请过的订单号不能再次申请退货
        int count=rejectMapper.countInstallOrder(rejectAddVo.getInstallOrder());
        if (count>0){
            throw new RuntimeException("已经申请过的安装单号不能再次申请退货");
        }
        String orderid= snowFlakeService.getNextId(0,0)+"";
        Reject reject= RejectConvert.INSTANCE.convertAddVo(rejectAddVo);
        reject.setRejectOrder(orderid);
        reject.setStatus(TauxiliarymaterialStatusEnum.已申请.name());
        this.commonUpdateService.UpdateColumns(reject,true);
        return new BaseResponse<>(this.rejectMapper.insertSelective(reject));
    }*/
    @PostMapping("/query")
    @ApiOperation(value = "退货列表查询")
    public BaseResponse<List<Delivery>> queryReject(Delivery Delivery){
        List<String> list= Arrays.asList("退货已申请","退货已驳回","退货已通过","退货完成");
        Delivery.setLogisticStatusList(list);
        return new BaseResponse(this.deliveryMapper.selectDeliveryList(Delivery));
    }
    @GetMapping("/detail")
    @ApiOperation(value = "退货详情")
    public BaseResponse<Delivery> getdetail(@RequestParam int id){
        /*Delivery delivery=this.deliveryMapper.selectByPrimaryKey(id);
        if (!StringUtil.isNullOrEmpty(delivery.getRejectMaterial())){
            String [] rejectMaterial=delivery.getRejectMaterial().split(",");
            List<String> list=new ArrayList<>();
            for (String rejectMaterials:rejectMaterial){
                list.add(rejectMaterials);
            }
            delivery.setRejectMaterialList(list);
        }*/
        return new BaseResponse(this.deliveryMapper.selectByPrimaryKey(id));
    }
   /* @GetMapping("/delete")
    @ApiOperation(value = "删除退货信息")
    public BaseResponse<Integer> delete(@RequestParam int id){
        return new BaseResponse(this.rejectMapper.deleteByPrimaryKey(id));
    }*/
    @PostMapping("/update")
    @ApiOperation("更新退货单信息")
    @ParamsValidate
    public BaseResponse<DeliveryUpdateDTO> update(@RequestBody @Validated(UpdateEntityGroup.class) DeliveryUpdateDTO deliveryUpdateDTO, BindingResult bindingResult){
        //var data=RejectConvert.INSTANCE.convertAddVo(rejectAddVo);
        this.commonUpdateService.UpdateColumns(deliveryUpdateDTO,false);
        deliveryUpdateDTO.setUpdatedTime(LocalDateTime.now());
        return new BaseResponse(this.deliveryMapper.updateByPrimaryKeySelective(deliveryUpdateDTO));
    }
    @PostMapping("/confirmReject")
    @ApiOperation("确认退货接口")
    @ParamsValidate
    public BaseResponse<Void> confirmReject(@RequestBody @Validated(UpdateEntityGroup.class) DeliveryUpdateDTO deliveryUpdateDTO, BindingResult bindingResult){
        //var data=RejectConvert.INSTANCE.convertAddVo(rejectAddVo);
        //String token = SpringContextUtil.getToken();
        Delivery before=this.deliveryMapper.selectByPrimaryKey(deliveryUpdateDTO.getId());
        Map<String, String> params = new HashMap<String, String>();
        params.put("warehouseId",before.getWarehouseId());
        params.put("materialsCode",before.getMaterialCode());
        params.put("quantity","1");
        //发货仓库库存恢复
        String jsonStr = HttpClientUtils.post(increaseUrl,params,"UTF-8", "UTF-8");
        JSONObject jsonObject=JSONObject.parseObject(jsonStr);
        String code=jsonObject.getString("code");
        if (StringUtil.isNullOrEmpty(code)){
            throw new ServiceException("发货仓库库存恢复失败");
        }
        //到货仓库库存扣减
        String jsonReduce = HttpClientUtils.post(reduceUrl,params,"UTF-8", "UTF-8");
        JSONObject jsonReduceObject=JSONObject.parseObject(jsonReduce);
        String codeReduce=jsonReduceObject.getString("code");
        if (StringUtil.isNullOrEmpty(codeReduce)){
            throw new ServiceException("到货仓库库存扣减失败");
        }
        deliveryUpdateDTO.setLogisticStatus("退货完成");
        deliveryUpdateDTO.setRejectTime(LocalDateTime.now());
        this.commonUpdateService.UpdateColumns(deliveryUpdateDTO,false);
        return new BaseResponse(this.deliveryMapper.updateByPrimaryKeySelective(deliveryUpdateDTO));
    }

    /*@PostMapping("/export")
    @ApiOperation(value = "导出退货单列表")
    public void export(HttpServletResponse response, @RequestBody Reject Reject) throws IOException {

//        ExcelUtil<WarehouseInfo> util = new ExcelUtil<WarehouseInfo>(WarehouseInfo.class);
//        util.exportExcel(response, list, "仓库信息管理数据");
        List<RejectExcelVo> data = RejectConvert.INSTANCE.convertExcelVoList(this
                .rejectMapper.selectRejectList(Reject));
        // 输出
        ExcelUtils.write(response, "退货管理列表.xls", "退货信息", RejectExcelVo.class, data);
    }*/
    @PostMapping("/check")
    @ApiOperation("审批退货单")
    @ParamsValidate
    public BaseResponse<Boolean> check(@RequestBody @Validated DeliveryUpdateDTO deliveryUpdateDTO,BindingResult bindingResult){
        var status=TauxiliarymaterialStatusEnum.getEnumByName(0,deliveryUpdateDTO.getLogisticStatus());
        if(status==null){
            throw new ServiceException(400,"status必须是枚举："+TauxiliarymaterialStatusEnum.getStatusTypeEnums(0).stream().map(e->e.name()).reduce("",(a,b)->a+","+b));
        }
        SessionInfo sessionInfo = tokenService.getUser();
        //var data=RejectConvert.INSTANCE.convertCheckVo(rejectCheckVo);
        deliveryUpdateDTO.setCheckUser(sessionInfo.getUserId());
        deliveryUpdateDTO.setCheckUserName(sessionInfo.getUserName());
        deliveryUpdateDTO.setCheckTime(LocalDateTime.now());
        this.commonUpdateService.UpdateColumns(deliveryUpdateDTO,false);
        return new BaseResponse(this.deliveryMapper.updateByPrimaryKeySelective(deliveryUpdateDTO));
    }


}
